package com.lzmispt.data.resource.service.resdata.impl;

package .tmp;

import com.alibaba.fastjson.JSONObject;
import com.lzmispt.core.client.MessageClient;
import com.lzmispt.core.entity.dto.authorize.UserInfoDTO;
import com.lzmispt.core.entity.dto.message.MessageEntity;
import com.lzmispt.data.center.entity.dto.resdata.ResDataDTO;
import com.lzmispt.data.center.entity.vo.resource.resdata.ResDataVO;
import com.lzmispt.data.resource.entity.dto.resanddata.ResAndDataEntityDTO;
import com.lzmispt.data.resource.manager.metadata.MetadataManager;
import com.lzmispt.data.resource.service.resdata.IResDataService;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.dcbase.log.CommonLogHelper;
import com.lzmispt.dcbase.resdata.ResDataHelper;
import com.lzmispt.dcbase.resdata.ResDataPO;
import com.lzmispt.dcbase.util.TokenUtil;
import com.lzmispt.framework.entity.entity.ResultEntity;
import com.lzmispt.framework.exception.ExceptionUtil;
import com.lzmispt.framework.log.LogHelper;
import com.lzmispt.framework.utils.WorkerId;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

@Service
public class ResDataServiceImpl implements IResDataService {
  @Autowired
  private MessageClient messageClient;
  
  public ResultEntity<Boolean> saveOrUpdateResData(List<ResDataDTO> resDataDTOList, String platCode, String tokenId) {
    for (ResDataDTO resDataDTO : resDataDTOList)
      saveOrUpdateResData(resDataDTO, platCode, tokenId); 
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  public Boolean saveOrUpdateResData(ResDataDTO resDataDTO, String platCode, String tokenId) {
    ResAndDataEntityDTO entity = MetadataManager.selectEntityByEntityId(resDataDTO.getEntityId(), DCConst.DataResourceCategory.RES, platCode, tokenId);
    ResDataPO resDataPO = new ResDataPO();
    BeanUtils.copyProperties(resDataDTO, resDataPO);
    ResDataPO existingResData = ResDataHelper.getResDataPOByIdEntity(resDataPO, entity.getEntityTableName());
    if (resDataDTO.getId() == null) {
      if (existingResData != null) {
        existingResData.setStructuredState(resDataDTO.getStructuredState());
        ResDataHelper.update(existingResData, entity.getEntityTableName());
        statusChangeSendMsg(existingResData, tokenId);
      } else {
        resDataPO.setId((Long)WorkerId.getNextId(Long.class));
        resDataPO.setCreateTime(new Date());
        resDataPO.setUpdateTime(new Date());
        resDataPO.setIsDelete(DCConst.FALSE);
        ResDataHelper.insert(resDataPO, entity.getEntityTableName());
        statusChangeSendMsg(resDataPO, tokenId);
      } 
    } else {
      ResDataHelper.update(resDataPO, entity.getEntityTableName());
      statusChangeSendMsg(resDataPO, tokenId);
    } 
    return Boolean.valueOf(true);
  }
  
  public ResultEntity<Boolean> deleteResDataById(ResDataDTO resDataDTO, String platCode, String tokenId) {
    ResAndDataEntityDTO entity = MetadataManager.selectEntityByEntityId(resDataDTO.getEntityId(), DCConst.DataResourceCategory.RES, platCode, tokenId);
    Boolean result = ResDataHelper.deleteResDataById(resDataDTO.getId(), entity.getEntityTableName(), entity.getEntityId());
    return ResultEntity.successMessage(result);
  }
  
  public ResultEntity<List<ResDataDTO>> getResDataCondition(ResDataDTO resDataDTO, String platCode, String tokenId) {
    ResAndDataEntityDTO entity = MetadataManager.selectEntityByEntityId(resDataDTO.getEntityId(), DCConst.DataResourceCategory.RES, platCode, tokenId);
    List<ResDataDTO> resDataDTOList = (List<ResDataDTO>)ResDataHelper.getResDataCondition(resDataDTO, entity.getEntityTableName()).stream().filter(Objects::nonNull).map(resDataPO -> {
          ResDataDTO resData = new ResDataDTO();
          BeanUtils.copyProperties(resDataPO, resData);
          return resData;
        }).collect(Collectors.toList());
    return ResultEntity.successMessage(resDataDTOList);
  }
  
  public void statusChangeSendMsg(ResDataPO resData, String tokenId) {
    try {
      UserInfoDTO userInfo = TokenUtil.getUserInfoByTokenId(tokenId);
      MessageEntity msg = new MessageEntity();
      msg.setId(String.valueOf(WorkerId.getNextId(Long.class)));
      JSONObject body = new JSONObject();
      body.put("msgCode", "res_data");
      JSONObject data = new JSONObject();
      data.put("entity_id", String.valueOf(resData.getEntityId()));
      data.put("lzdc_sys_dbid", resData.getDataId());
      data.put("structured_state", resData.getStructuredState());
      body.put("obj", data);
      msg.setBody(body.toJSONString());
      msg.setApplication("dc_msg_update_structured_state");
      msg.setOrganization(String.valueOf(userInfo.getOrganizationId()));
      msg.setUser(String.valueOf(userInfo.getUserId()));
      this.messageClient.sendTopicMessage(msg);
      CommonLogHelper.consoleInfo("发送消息下资源规整状态" + body.toJSONString(), DCConst.ServerClientType.RESOURCE);
    } catch (Exception e) {
      LogHelper.error("规整状态变化推送前端消息异常", ExceptionUtil.getStackTraceMsg(e));
    } 
  }
  
  public ResultEntity<ResDataVO> getResDataSingleRecordByQuery(ResDataDTO resDataDTO, String platCode, String tokenId) {
    ResAndDataEntityDTO entity = MetadataManager.selectEntityByEntityId(resDataDTO.getEntityId(), DCConst.DataResourceCategory.RES, platCode, tokenId);
    ResDataPO resDataPO = ResDataHelper.getResDataSingleRecordByQuery(resDataDTO, entity.getEntityTableName());
    ResDataVO resDataVO = new ResDataVO();
    if (!ObjectUtils.isEmpty(resDataPO))
      BeanUtils.copyProperties(resDataVO, resDataPO); 
    return ResultEntity.successMessage(resDataVO);
  }
}
